import React from 'react';
import { type Store } from 'redux';
import { Match, type ConnectedRouterProps, type CreateRenderOptions, type MatchBase, type RenderArgs, type Resolver, type Router } from './typeUtils';
interface CreateProps extends CreateRenderOptions {
    render?: (args: RenderArgs) => React.ReactElement;
}
interface BaseRouterProps extends ConnectedRouterProps {
    store: Store;
    match: MatchBase;
    resolvedMatch: MatchBase;
}
interface BaseRouterState {
    isInitialRender: boolean;
    match: MatchBase;
    matchContext: any;
    resolver: Resolver;
    iteration: number;
    routerContext: {
        match: Match<any> | null;
        router: Router;
    };
    element: React.ReactElement | null;
}
export default function createBaseRouter({ renderPending, renderReady, renderError, 
/**
 * The Router level render, is responsible for turning an array of route elements
 * into a single composed element that can be rendered by React.
 *
 * Turning:
 *
 * ```jsx
 * [<AppPage>, null, <ProductPage>, <ProductHistoryPage />]
 * ```
 * Into:
 *
 * ```jsx
 * <AppPage>
 *  <ProductPage>
 *    <ProductHistoryPage />
 *  </ProductPage>
 * </AppPage>
 * ```
 */
render, }: CreateProps): React.ComponentClass<BaseRouterProps, BaseRouterState>;
export {};
