"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createBaseRouter;
var _dequal = require("dequal");
var _react = _interopRequireDefault(require("react"));
var _tinyWarning = _interopRequireDefault(require("tiny-warning"));
var _ActionTypes = _interopRequireDefault(require("./ActionTypes"));
var _RouterContext = _interopRequireDefault(require("./RouterContext"));
var _StaticContainer = _interopRequireDefault(require("./StaticContainer"));
var _createRender = _interopRequireDefault(require("./createRender"));
var _createStoreRouterObject = _interopRequireDefault(require("./createStoreRouterObject"));
var _resolveRenderArgs = _interopRequireDefault(require("./resolveRenderArgs"));
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function createBaseRouter({
  renderPending,
  renderReady,
  renderError,
  /**
   * The Router level render, is responsible for turning an array of route elements
   * into a single composed element that can be rendered by React.
   *
   * Turning:
   *
   * ```jsx
   * [<AppPage>, null, <ProductPage>, <ProductHistoryPage />]
   * ```
   * Into:
   *
   * ```jsx
   * <AppPage>
   *  <ProductPage>
   *    <ProductHistoryPage />
   *  </ProductPage>
   * </AppPage>
   * ```
   */
  render = (0, _createRender.default)({
    renderPending,
    renderReady,
    renderError
  })
}) {
  class BaseRouter extends _react.default.Component {
    constructor(props) {
      super(props);
      const {
        store,
        match,
        matchContext,
        resolver,
        initialRenderArgs
      } = props;
      this.router = (0, _createStoreRouterObject.default)(store);
      this.state = {
        isInitialRender: true,
        match,
        matchContext,
        resolver,
        iteration: 0,
        routerContext: {
          router: this.router,
          match: initialRenderArgs || null
        },
        element: initialRenderArgs ? render(initialRenderArgs) : null
      };
      this.lastIteration = 0;
      this.pendingResolvedMatch = false;
      this.dispatchMatch = pendingMatch => {
        store.dispatch({
          type: _ActionTypes.default.RESOLVE_MATCH,
          payload: pendingMatch
        });
      };
    }

    // We use componentDidMount and componentDidUpdate to resolve the match if
    //  needed because element resolution is asynchronous anyway, and this lets
    //  us not worry about setState not being available in the constructor, or
    //  about having to pass around nextProps.

    componentDidMount() {
      this.mounted = true;
      if (!this.props.initialRenderArgs) {
        this.resolveMatch();
      }
      if (process.env.NODE_ENV !== "production" && typeof window !== 'undefined') {
        /* eslint-env browser */

        if (window.__FOUND_HOT_RELOAD__) {
          process.env.NODE_ENV !== "production" ? (0, _tinyWarning.default)(!window.__FOUND_REPLACE_ROUTE_CONFIG__, 'Replacing existing hot reloading hook. Do not render more than one router instance when using hot reloading.') : void 0;
          window.__FOUND_REPLACE_ROUTE_CONFIG__ = this.router.replaceRouteConfig;
        }

        /* eslint-env browser: false */
      }
    }
    static getDerivedStateFromProps({
      match,
      resolver,
      matchContext
    }, state) {
      if (state.isInitialRender) {
        return {
          isInitialRender: false
        };
      }
      if (match !== state.match || resolver !== state.resolver || !(0, _dequal.dequal)(matchContext, state.matchContext)) {
        return {
          match,
          resolver,
          matchContext,
          iteration: state.iteration + 1
        };
      }
      return null;
    }
    componentDidUpdate() {
      if (this.state.iteration > this.lastIteration) {
        this.lastIteration = this.state.iteration;
        this.resolveMatch();
      }
    }
    componentWillUnmount() {
      this.mounted = false;
      if (process.env.NODE_ENV !== "production" && typeof window !== 'undefined') {
        /* eslint-env browser */

        if (window.__FOUND_HOT_RELOAD__) {
          delete window.__FOUND_REPLACE_ROUTE_CONFIG__;
        }

        /* eslint-env browser: false */
      }
    }
    async resolveMatch() {
      const pendingIteration = this.lastIteration;
      const pendingMatch = this.props.match;
      try {
        for await (const renderArgs of (0, _resolveRenderArgs.default)(this.router,
        // TODO: this should be removed once resolveRenderArgs is in TS
        this.props)) {
          // Don't do anything if we're resolving an outdated match.
          if (!this.mounted || this.lastIteration !== pendingIteration) {
            return;
          }

          // If we're about to mark the match resolved, delay the rerender
          //  until we do so.
          this.pendingResolvedMatch = !!((renderArgs.elements || renderArgs.error) && this.props.resolvedMatch !== pendingMatch);
          this.setState({
            routerContext: {
              router: this.router,
              match: renderArgs
            },
            element: render(renderArgs)
          });
          if (this.pendingResolvedMatch) {
            // If this is a new match, update the store, so we can rerender at
            //  the same time as all of the links and other components
            //  connected to the router state.
            this.pendingResolvedMatch = false;
            this.dispatchMatch(pendingMatch);
          }
        }
      } catch (e) {
        if (!this.mounted || this.lastIteration !== pendingIteration) {
          return;
        }
        if (e.isFoundRedirectException) {
          this.router.replace(e.location);
          return;
        }

        /* istanbul ignore next: paranoid guard */
        throw e;
      }
    }
    render() {
      const {
        iteration,
        routerContext,
        element
      } = this.state;

      // Don't rerender synchronously if we have another rerender coming.
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(_StaticContainer.default, {
        shouldUpdate: this.lastIteration === iteration && !this.pendingResolvedMatch,
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_RouterContext.default.Provider, {
          value: routerContext,
          children: element
        })
      });
    }
  }
  return BaseRouter;
}