import React from 'react';
import { Unresolved } from './ResolverUtils';
import { type ResolvedElement, type RouteMatch } from './typeUtils';
/**
 * maps an array of `Route`s to React elements. The returned array
 * may contain elements, `null`, or `undefined`. The nullish values produce different results!
 * `undefined` means "still resolving", whereas `null` means "nothing to render". This
 * is important because the resolver calling this, will not update the existing UI
 * if there is an `undefined` value, whereas it will update `null`s rendering nothing.
 *
 * @param {*} routeMatches An array of matched routes
 * @param {*} Components An array of possibly resolved route Components
 * @param {*} matchData An array of possibly resolved route data
 * @returns [null | ReactElement | undefined]
 */
export default function createElements(routeMatches: Array<RouteMatch>, Components: (React.ComponentType<any> | Unresolved | undefined)[], matchData: any): Array<ResolvedElement | undefined>;
