"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createFarceRouter;
var _useIsomorphicEffect = _interopRequireDefault(require("@restart/hooks/useIsomorphicEffect"));
var _react = require("react");
var _createBaseRouter = _interopRequireDefault(require("./createBaseRouter"));
var _createFarceStore = _interopRequireDefault(require("./createFarceStore"));
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function createFarceRouter({
  store: userStore,
  historyProtocol,
  historyMiddlewares,
  historyOptions,
  routeConfig,
  // @ts-expect-error TODO: matcher options should not accessible to end user
  matcherOptions,
  getFound = ({
    found
  }) => found,
  ...options
}) {
  const Router = (0, _createBaseRouter.default)(options);
  const store = userStore || (0, _createFarceStore.default)({
    historyProtocol,
    historyMiddlewares,
    historyOptions,
    routeConfig,
    matcherOptions
  });
  const FarceRouterInstance = /*#__PURE__*/(0, _react.forwardRef)((props, ref) => {
    const [state, setState] = (0, _react.useState)(() => {
      const {
        match,
        resolvedMatch
      } = getFound(store.getState());
      return {
        match,
        resolvedMatch
      };
    });
    (0, _useIsomorphicEffect.default)(() => {
      return store.subscribe(() => {
        setState(prev => {
          const {
            match,
            resolvedMatch
          } = getFound(store.getState());
          if (prev?.match === match && prev.resolvedMatch === resolvedMatch) {
            return prev;
          }
          return {
            match,
            resolvedMatch
          };
        });
      });
    }, []);
    (0, _react.useImperativeHandle)(ref, () => store, []);
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(Router, {
      ...props,
      ...state,
      store: store
    });
  });
  FarceRouterInstance.displayName = 'FarceRouter';
  return FarceRouterInstance;
}