"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RouterProvider = RouterProvider;
exports.getFarceResult = getFarceResult;
var _Actions = _interopRequireDefault(require("farce/Actions"));
var _ServerProtocol = _interopRequireDefault(require("farce/ServerProtocol"));
var _react = _interopRequireWildcard(require("react"));
var _RouterContext = _interopRequireDefault(require("./RouterContext"));
var _createFarceStore = _interopRequireDefault(require("./createFarceStore"));
var _createRender = _interopRequireDefault(require("./createRender"));
var _getStoreRenderArgs = _interopRequireDefault(require("./getStoreRenderArgs"));
var _resolver = _interopRequireDefault(require("./resolver"));
var _jsxRuntime = require("react/jsx-runtime");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function RouterProvider({
  renderArgs,
  children
}) {
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_RouterContext.default.Provider, {
    value: (0, _react.useMemo)(() => ({
      router: renderArgs.router,
      match: renderArgs
    }), [renderArgs]),
    children: children
  });
}
async function getFarceResult({
  url,
  historyMiddlewares,
  historyOptions,
  routeConfig,
  matchContext,
  resolver = _resolver.default,
  renderPending,
  renderReady,
  renderError,
  render = (0, _createRender.default)({
    renderPending,
    renderReady,
    renderError
  })
}) {
  const store = (0, _createFarceStore.default)({
    historyProtocol: new _ServerProtocol.default(url),
    historyMiddlewares,
    historyOptions,
    routeConfig
  });
  let renderArgs;
  try {
    renderArgs = await (0, _getStoreRenderArgs.default)({
      store,
      matchContext,
      resolver
    });
  } catch (e) {
    if (e.isFoundRedirectException) {
      // The store is not exposed to the user, so we need to build the redirect
      // URL here.
      return {
        status: e.status,
        redirect: {
          url: store.farce.createHref(e.location)
        }
      };
    }

    /* istanbul ignore next: paranoid guard */
    throw e;
  } finally {
    // This is a no-op with ServerProtocol, but it doesn't hurt.
    store.dispatch(_Actions.default.dispose());
  }
  return {
    status: 'error' in renderArgs && renderArgs.error ? renderArgs.error.status : 200,
    element: /*#__PURE__*/(0, _jsxRuntime.jsx)(RouterProvider, {
      renderArgs: renderArgs,
      children: render(renderArgs)
    })
  };
}