import { type IsActiveOptions, type LocationDescriptorObject, type Match, type MatchBase, type MatcherResult, type Params, type ParamsDescriptor, type Query, type QueryDescriptor, type RouteConfig, type RouteIndices, type RouteObject } from './typeUtils';
export type RouteConfigGroups = Record<string, RouteConfig>;
export type IntermediateRouteMatch = {
    index: number;
    params: Params;
} | {
    groups: Record<string, IntermediateRouteMatch[]>;
};
export interface MatcherOptions {
    warnOnPotentialMissingIndexRoutes?: boolean;
    warnOnPartiallyMatchedNamedRoutes?: boolean;
}
export default class Matcher {
    private routeConfig;
    private options;
    constructor(routeConfig: RouteConfig, options?: MatcherOptions);
    match({ pathname }: {
        pathname: string;
    }): MatcherResult | null;
    getRoutes({ routeIndices }: MatchBase): RouteObject[] | null;
    protected matchRoutes(routeConfig: RouteConfig, pathname: string): null | IntermediateRouteMatch[];
    protected matchRoute(route: RouteObject, pathname: string): {
        params: Params;
        remaining: string;
    } | null;
    protected matchGroups(routeGroups: Record<string, RouteConfig>, pathname: string, parentPathname: string): Record<string, IntermediateRouteMatch[]> | null;
    getCanonicalPattern(pattern: string): string;
    protected makePayload(matches: IntermediateRouteMatch[]): MatcherResult;
    protected getRoutesFromIndices(routeIndices: RouteIndices, routeConfigOrGroups: RouteConfig | Record<string, RouteConfig> | undefined): RouteObject[];
    joinPaths(basePath: string, path?: string | null): string;
    isActive({ location: matchLocation }: Match, location: LocationDescriptorObject, { exact }?: IsActiveOptions): boolean;
    format(pattern: string, params: ParamsDescriptor): string;
    isPathnameActive(matchPathname: string, pathname: string, exact?: boolean): boolean;
    isQueryActive(matchQuery: Query, query?: QueryDescriptor | undefined | null): boolean;
    replaceRouteConfig(routeConfig: RouteConfig): void;
}
